@@ -4,29 +4,33 @@
 # Try to assign tips to a pre-existing tree based on a taxonomy
 # Jonathan Chang, May 13, 2016
 
-# Pragmata
 from __future__ import division
 from __future__ import print_function
 
-# Internal
-from .lib import get_birth_death_rates, get_ages, is_binary, get_short_branches, get_tip_labels, crown_capture_probability, edge_iter, get_new_times, ensure_tree_node_depths
-from . import fastmrca
-
-# Python standard library
+import collections
+import csv
 import logging
 import operator
 import random
 import sys
 from time import time
-import collections
-import csv
 
-# Third party
-import dendropy
 import click
+import dendropy
+
+from . import fastmrca
+from .lib import crown_capture_probability
+from .lib import edge_iter
+from .lib import ensure_tree_node_depths
+from .lib import get_ages
+from .lib import get_birth_death_rates
+from .lib import get_new_times
+from .lib import get_short_branches
+from .lib import get_tip_labels
+from .lib import is_binary
 
 logger = logging.getLogger(__name__)
-# Speed up logging for pypy
+# Speed up logging for PyPy
 logging._srcfile = None
 logging.logThreads = 0
 logging.logProcesses = 0
@@ -44,26 +48,32 @@ def search_ancestors_for_valid_backbone_node(taxonomy_node, backbone_tips, ccp):
     seen = []
     for anc in taxonomy_node.ancestor_iter():
         if anc.label in invalid_map:
-            logger.debug("Cache HIT on invalid_map for {} ({} => {})".format(taxonomy_node.label, anc.label, invalid_map[anc.label].label))
+            logger.debug(
+                f"Cache HIT on invalid_map for {taxonomy_node.label} ({anc.label} => {invalid_map[anc.label].label})"
+            )
             anc = invalid_map[anc.label]
         full_tax = get_tip_labels(anc)
         extant_tax = full_tax.intersection(backbone_tips)
         backbone_node = fastmrca.get(extant_tax)
         seen.append(anc.label)
+        computed_ccp = crown_capture_probability(len(full_tax), len(extant_tax))
         if backbone_node is None:
-            logger.info("    {}: ancestor {} not monophyletic!".format(taxonomy_node.label, anc.label))
-        elif crown_capture_probability(len(full_tax), len(extant_tax)) < ccp:
-            logger.info("    {}: ancestor {} fails crown threshold ({} < {}); using stem".format(taxonomy_node.label, anc.label, crown_capture_probability(len(full_tax), len(extant_tax)), ccp))
+            logger.info(f"    {taxonomy_node.label}: ancestor {anc.label} not monophyletic!")
+        elif computed_ccp < ccp:
+            logger.info(
+                f"    {taxonomy_node.label}: ancestor {anc.label} fails crown threshold ({computed_ccp} < {ccp}); using stem"
+            )
             taxonomy_target = anc
             backbone_target = backbone_node.parent_node
             break
         else:
             taxonomy_target = anc
             backbone_target = backbone_node
-            logger.info("    {}: will instead assign these taxa to {}".format(taxonomy_node.label, taxonomy_target.label))
+            logger.info(f"    {taxonomy_node.label}: will instead assign these taxa to {taxonomy_target.label}")
             break
     else:
-        logger.error("Couldn't find valid taxonomy node in ancestor chain for {} ({})".format(taxonomy_node.label, " => ".join(seen)))
+        anc_chain = " => ".join(seen)
+        logger.error(f"Couldn't find valid taxonomy node in ancestor chain for {taxonomy_node.label} ({anc_chain})")
         sys.exit(1)
     seen.pop()  # ignore last node
     for x in seen:
@@ -71,7 +81,9 @@ def search_ancestors_for_valid_backbone_node(taxonomy_node, backbone_tips, ccp):
     return (taxonomy_target, backbone_target)
 
 
-def get_new_branching_times(backbone_node, taxonomy_node, backbone_tree, told=None, tyoung=0, min_ccp=0.8, num_new_times=None):
+def get_new_branching_times(
+    backbone_node, taxonomy_node, backbone_tree, told=None, tyoung=0, min_ccp=0.8, num_new_times=None
+):
     """
     Get `n_total` new branching times for a `node`.
     """
@@ -82,13 +94,15 @@ def get_new_branching_times(backbone_node, taxonomy_node, backbone_tree, told=No
         if backbone_node.parent_node:
             new_told = backbone_node.parent_node.age
             if told is not None:
-                logger.debug("    {}: tmax {} => {} because ccp {:.2f} < {}".format(taxon, told, new_told, ccp, min_ccp))
+                logger.debug(f"    {taxon}: tmax {told} => {new_told} because ccp {new_told:.2f} < {min_ccp}")
             else:
-                logger.debug("    {}: tmax set to {} because ccp {:.2f} < {}".format(taxon, new_told, ccp, min_ccp))
+                logger.debug(f"    {taxon}: tmax set to {new_told} because ccp {ccp:.2f} < {min_ccp}")
         else:
             # TODO: check for a root edge and graft a fake node above that
             new_told = backbone_node.age
-            logger.debug("    {}: tmax set to {} because even though ccp {:.2f} < {} clade is tree root".format(taxon, new_told, ccp, min_ccp))
+            logger.debug(
+                f"    {taxon}: tmax set to {new_told} because even though ccp {ccp:.2f} < {min_ccp} clade is tree root"
+            )
         told = new_told
     n_extant = len(backbone_node.leaf_nodes())
     n_total = len(taxonomy_node.leaf_nodes())
@@ -98,16 +112,18 @@ def get_new_branching_times(backbone_node, taxonomy_node, backbone_tree, told=No
     if len(backbone_node.leaf_nodes()) == 1 and told is None:
         # attach to stem in the case of a singleton
         told = backbone_node.parent_node.age
-        logger.debug("    {}: tmax set to {:.2f} because taxon is singleton".format(taxon, told))
+        logger.debug(f"    {taxon}: tmax set to {told:.2f} because taxon is singleton")
     if told is None:
         told = max(ages)
-        logger.debug("    {}: tmax set to {:.2f} because of max age".format(taxon, told))
-    logger.debug("    {}: {} new times: b={:.2f}, d={:.2f}, tmax={:.2f}, tmin={:.2f}".format(taxon, num_new_times, birth, death, told, tyoung))
+        logger.debug(f"    {taxon}: tmax set to {told:.2f} because of max age")
+    logger.debug(
+        f"    {taxon}: {num_new_times} new times: b={birth:.2f}, d={death:.2f}, tmax={told:.2f}, tmin={tyoung:.2f}"
+    )
     times = get_new_times(ages, birth, death, num_new_times, told, tyoung)
     if len(times) > 5:
-        logger.debug("    {}: {:.2f}..{:.2f}".format(taxon, times[0], times[-1]))
+        logger.debug(f"    {taxon}: {times[0]:.2f}..{times[-1]:.2f}")
     else:
-        logger.debug(("    {}: " + ", ".join(["{:.2f}" for x in times])).format(taxon, *times))
+        logger.debug(f"    {taxon}: " + ", ".join(["{:.2f}" for x in times]).format(*times))
     return times
 
 
@@ -121,8 +137,9 @@ def fill_new_taxa(namespace, node, new_taxa, times, stem=False, excluded_nodes=N
         new_leaf.age = 0
         node = graft_node(node, new_node, stem)
 
-    if list(get_short_branches(node)):
-        logger.info("{} short branches detected".format(len(list(get_short_branches(node)))))
+    count_short_branches = len(list(get_short_branches(node)))
+    if count_short_branches:
+        logger.info(f"{count_short_branches} short branches detected")
 
     return node
 
@@ -149,6 +166,7 @@ def graft_node(graft_recipient, graft, stem=False):
     # 4. Must not be locked (intruding on monophyly)
     def filter_fn(x):
         return x.head_node.age <= graft.age and x.head_node.parent_node.age >= graft.age and x.label != "locked"
+
     all_edges = list(edge_iter(graft_recipient))
     if stem:
         # also include the crown node's subtending edge
@@ -156,7 +174,7 @@ def graft_node(graft_recipient, graft, stem=False):
     eligible_edges = [x for x in all_edges if filter_fn(x)]
 
     if not eligible_edges:
-        raise Exception("could not place node {} in clade {}".format(graft, graft_recipient))
+        raise Exception(f"could not place node {graft} in clade {graft_recipient}")
     focal_node = random.choice([x.head_node for x in eligible_edges])
     seed_node = focal_node.parent_node
     sisters = focal_node.sibling_nodes()
@@ -232,7 +250,7 @@ def lock_clade(node):
         edge.label = "locked"
     post = count_locked(node)
     if pre != post:
-        logger.debug("locking clade: {} => {}".format(pre, post))
+        logger.debug(f"locking clade: {pre} => {post}")
 
 
 def count_locked(node):
@@ -250,7 +268,9 @@ def get_min_age(node):
         return 0.0
 
 
-def process_node(backbone_tree, backbone_bitmask, all_possible_tips, taxon_node, min_ccp, default_birth, default_death, yule=False):
+def process_node(
+    backbone_tree, backbone_bitmask, all_possible_tips, taxon_node, min_ccp, default_birth, default_death, yule=False
+):
     # TODO: Fix all the returns and refactor this into something sane
     global mrca_rates
     taxon = taxon_node.label
@@ -263,37 +283,39 @@ def process_node(backbone_tree, backbone_bitmask, all_possible_tips, taxon_node,
         parent = "ROOT"
     species = get_tip_labels(taxon_node)
     if not taxon:
-        logger.debug("MRCA: skipping unlabeled rank with {} species".format(len(species)))
+        logger.debug(f"MRCA: skipping unlabeled rank with {len(species)} species")
         return
     all_bitmask = backbone_tree.taxon_namespace.taxa_bitmask(labels=species)
     extant_bitmask = all_bitmask & backbone_bitmask
     if extant_bitmask is None or extant_bitmask == 0:
-        logger.debug("MRCA: {} not present in backbone".format(taxon))
-        mrca_rates[taxon] = (birth, death, 0.0, "from {} (unsampled)".format(parent))
+        logger.debug(f"MRCA: {taxon} not present in backbone")
+        mrca_rates[taxon] = (birth, death, 0.0, f"from {parent} (unsampled)")
         return
     mrca = backbone_tree.mrca(leafset_bitmask=extant_bitmask)
     if not species.issuperset(get_tip_labels(mrca)):
-        logger.debug("MRCA: {} not monophyletic in backbone".format(taxon))
-        mrca_rates[taxon] = (birth, death, 0.0, "from {} (not monophyletic)".format(parent))
+        logger.debug(f"MRCA: {taxon} not monophyletic in backbone")
+        mrca_rates[taxon] = (birth, death, 0.0, f"from {parent} (not monophyletic)")
         return
     extant = len(mrca.leaf_nodes())
     total = len(taxon_node.leaf_nodes())
     if extant > total:
-        logger.warning("MRCA: {} has {} extant species but should have {} total species".format(taxon, extant, total))
-        mrca_rates[taxon] = (birth, death, 0, "from {} (extant exceeds total)".format(parent))
+        logger.warning(f"MRCA: {taxon} has {extant} extant species but should have {total} total species")
+        mrca_rates[taxon] = (birth, death, 0, f"from {parent} (extant exceeds total)")
         return
     ccp = crown_capture_probability(total, extant)
     if total == 1:
-        logger.debug("MRCA: {} is a singleton".format(taxon))
-        mrca_rates[taxon] = (birth, death, ccp, "from {} (singleton)".format(parent))
+        logger.debug(f"MRCA: {taxon} is a singleton")
+        mrca_rates[taxon] = (birth, death, ccp, f"from {parent} (singleton)")
         return
     if ccp < min_ccp:
-        logger.debug("MRCA: {} has crown capture probability {:.2f} < {:.2f} ({}/{} species)".format(taxon, ccp, min_ccp, extant, total))
-        mrca_rates[taxon] = (birth, death, ccp, "from {} (crown capture probability)".format(parent))
+        logger.debug(
+            f"MRCA: {taxon} has crown capture probability {ccp:.2f} < {min_ccp:.2f} ({extant}/{total} species)"
+        )
+        mrca_rates[taxon] = (birth, death, ccp, f"from {parent} (crown capture probability)")
         return
     sf = extant / total
     birth, death = get_birth_death_rates(mrca, sf, yule)
-    logger.debug("MRCA: {} b={}, d={}, sf={}, ccp={}".format(taxon, birth, death, sf, ccp))
+    logger.debug(f"MRCA: {taxon} b={birth}, d={death}, sf={sf}, ccp={ccp}")
     mrca_rates[taxon] = (birth, death, ccp, "computed")
 
 
@@ -310,16 +332,27 @@ def run_precalcs(taxonomy_tree, backbone_tree, min_ccp=0.8, min_extant=3, yule=F
     logger.debug("Computing root birth and death rates.")
     extant_bitmask = backbone_bitmask & backbone_tree.taxon_namespace.taxa_bitmask(labels=all_possible_tips)
     root_mrca = backbone_tree.mrca(leafset_bitmask=extant_bitmask)
-    root_birth, root_death = get_birth_death_rates(root_mrca, len(root_mrca.leaf_nodes()) / len(all_possible_tips), yule)
+    root_birth, root_death = get_birth_death_rates(
+        root_mrca, len(root_mrca.leaf_nodes()) / len(all_possible_tips), yule
+    )
 
-    with click.progressbar(taxonomy_tree.preorder_internal_node_iter(exclude_seed_node=True), width=12, label="Rates", length=nnodes, show_pos=True, item_show_func=lambda x: x.label if x else None) as progress:
+    with click.progressbar(
+        taxonomy_tree.preorder_internal_node_iter(exclude_seed_node=True),
+        width=12,
+        label="Rates",
+        length=nnodes,
+        show_pos=True,
+        item_show_func=lambda x: x.label if x else None,
+    ) as progress:
         for node in progress:
             # updates global mrca_rates as a side effect
-            process_node(backbone_tree, backbone_bitmask, all_possible_tips, node, min_ccp, root_birth, root_death, yule)
+            process_node(
+                backbone_tree, backbone_bitmask, all_possible_tips, node, min_ccp, root_birth, root_death, yule
+            )
 
     diff = time() - start_time
     if diff > 1:
-        logger.debug("FastMRCA calculation time: {:.1f} seconds".format(diff))
+        logger.debug(f"FastMRCA calculation time: {diff:.1f} seconds")
     return mrca_rates
 
 
@@ -330,13 +363,16 @@ def update_tree_view(tree):
     return get_tip_labels(tree)
 
 
-
 @click.command()
 @click.option("--taxonomy", help="a taxonomy tree", type=click.File("r"), required=True)
-@click.option("--backbone", help="the backbone tree to attach the taxonomy tree to", type=click.File("r"), required=True)
+@click.option(
+    "--backbone", help="the backbone tree to attach the taxonomy tree to", type=click.File("r"), required=True
+)
 @click.option("--outgroups", help="comma separated list of outgroup taxa to ignore")
 @click.option("--output", required=True, help="output base name to write out")
-@click.option("--min-ccp", help="minimum probability to use to say that we've sampled the crown of a clade", default=0.8)
+@click.option(
+    "--min-ccp", help="minimum probability to use to say that we've sampled the crown of a clade", default=0.8
+)
 @click.option("--yule", help="assume a Yule pure-birth model (force extinction to be 0)", default=False, is_flag=True)
 @click.option("-v", "--verbose", help="emit extra information (can be repeated)", count=True)
 def main(taxonomy, backbone, outgroups, output, min_ccp, verbose, yule):
@@ -352,7 +388,7 @@ def main(taxonomy, backbone, outgroups, output, min_ccp, verbose, yule):
         logger.setLevel(logging.WARNING)
         logger.addHandler(logging.StreamHandler())
 
-    logger.info("Reading taxonomy".format(taxonomy))
+    logger.info("Reading taxonomy")
     taxonomy = dendropy.Tree.get_from_stream(taxonomy, schema="newick", rooting="default-rooted")
     tn = taxonomy.taxon_namespace
     tn.is_mutable = True
@@ -366,13 +402,14 @@ def main(taxonomy, backbone, outgroups, output, min_ccp, verbose, yule):
     if msg:
         logger.warning(msg)
 
-    logger.info("Reading backbone".format(backbone))
+    logger.info("Reading backbone")
 
     try:
         tree = dendropy.Tree.get_from_stream(backbone, schema="newick", rooting="default-rooted", taxon_namespace=tn)
     except dendropy.utility.error.ImmutableTaxonNamespaceError as e:
-        logger.error("DendroPy error: {}".format(e))
-        print("""
+        logger.error(f"DendroPy error: {e}")
+        print(
+            """
 This usually indicates your backbone has species that are not present in your
 taxonomy. Outgroups not in the taxonomy can be excluded with the argument:
 
@@ -381,7 +418,8 @@ taxonomy. Outgroups not in the taxonomy can be excluded with the argument:
 For more details, run:
 
     tact_add_taxa --help
-""")
+"""
+        )
         sys.exit(1)
 
     if not is_binary(tree):
@@ -394,7 +432,7 @@ For more details, run:
     tree_tips = get_tip_labels(tree)
     all_possible_tips = get_tip_labels(taxonomy)
 
-    logger.info("Backbone needs to add {} tips".format(len(tree_tips.symmetric_difference(all_possible_tips))))
+    logger.info(f"Backbone needs to add {len(tree_tips.symmetric_difference(all_possible_tips))} tips")
 
     full_clades = set()
 
@@ -410,11 +448,13 @@ For more details, run:
 
     initial_length = len(tree_tips)
 
-    bar = click.progressbar(label="TACT",
-                            length=len(all_possible_tips) - initial_length,
-                            show_pos=True,
-                            width=12,
-                            item_show_func=lambda x: x)
+    bar = click.progressbar(
+        label="TACT",
+        length=len(all_possible_tips) - initial_length,
+        show_pos=True,
+        width=12,
+        item_show_func=lambda x: x,
+    )
 
     def bar_update():
         bar.pos = len(tree_tips) - initial_length
@@ -427,14 +467,16 @@ For more details, run:
             continue
         species = get_tip_labels(taxon_node)
         extant_species = tree_tips.intersection(species)
-        logger.info("**  {} ({}/{})  **".format(taxon, len(extant_species), len(species)))
+        logger.info(f"**  {taxon} ({len(extant_species)}/{len(species)})  **")
         ccp = mrca_rates[taxon][2]
 
-        clades_to_generate = full_clades.intersection([x.label for x in taxon_node.postorder_internal_node_iter(exclude_seed_node=True)])
+        clades_to_generate = full_clades.intersection(
+            [x.label for x in taxon_node.postorder_internal_node_iter(exclude_seed_node=True)]
+        )
 
         if not extant_species:
             # No species sampled, so create a clade from whole cloth
-            logger.debug("    {}: no species sampled, will create later".format(taxon))
+            logger.debug(f"    {taxon}: no species sampled, will create later")
             full_clades.add(taxon)
             bar_update()
             continue
@@ -442,19 +484,19 @@ For more details, run:
         # Check for monophyly for this node
         node = fastmrca.get(extant_species)
         if not node:
-            logger.info("    {}: is not monophyletic".format(taxon))
+            logger.info(f"    {taxon}: is not monophyletic")
             continue
 
         if extant_species == species:
             # Everything sampled and monophyletic, so skip this
             lock_clade(node)
-            logger.debug("    {}: all species accounted for".format(taxon))
+            logger.debug(f"    {taxon}: all species accounted for")
             continue
 
         if tree_tips.issuperset(species):
             # XXX: Does this check ever get triggered?
             lock_clade(node)
-            logger.info("    {}: all species already present in tree".format(taxon))
+            logger.info(f"    {taxon}: all species already present in tree")
             continue
 
         clade_ranks = [(clade, taxonomy.find_node_with_label(clade).level()) for clade in clades_to_generate]
@@ -467,17 +509,27 @@ For more details, run:
             full_node = taxonomy.find_node_with_label(clade)
             full_node_species = get_tip_labels(full_node)
             if tree_tips.issuperset(full_node_species):
-                logger.info("    {}: skipping clade {} as all species already present in tree".format(taxon, clade))
+                logger.info(f"    {taxon}: skipping clade {clade} as all species already present in tree")
                 full_clades.remove(clade)
                 continue
-            logger.info("    {}: adding clade {} (n={})".format(taxon, clade, len(full_node.leaf_nodes())))
+            logger.info(f"    {taxon}: adding clade {clade} (n={len(full_node.leaf_nodes())})")
             # Generate all times needed to attach to the main clade
-            times = get_new_branching_times(node, taxon_node, tree, tyoung=0, min_ccp=min_ccp, num_new_times=len(full_node_species))
+            times = get_new_branching_times(
+                node, taxon_node, tree, tyoung=0, min_ccp=min_ccp, num_new_times=len(full_node_species)
+            )
 
             if is_fully_locked(node):
-                logger.info("    {}: is fully locked, so attaching to stem".format(taxon))
+                logger.info(f"    {taxon}: is fully locked, so attaching to stem")
                 # Must attach to stem for this clade, so generate a time on the stem lineage
-                times2 = get_new_branching_times(node, taxon_node, tree, min_ccp=min_ccp, told=node.parent_node.age, tyoung=node.age, num_new_times=1)
+                times2 = get_new_branching_times(
+                    node,
+                    taxon_node,
+                    tree,
+                    min_ccp=min_ccp,
+                    told=node.parent_node.age,
+                    tyoung=node.age,
+                    num_new_times=1,
+                )
                 # Drop the oldest time and add on our new time on the stem lineage
                 times.sort()
                 times.pop()
@@ -486,8 +538,12 @@ For more details, run:
                 # Even if the main clade isn't fully locked, it might have a constraint on a valid attachment point
                 min_age = get_min_age(node)
                 if min_age > 0 and max(times) < min_age:
-                    logger.info("    {}: has a minimum age constraint {:.2f} but oldest generated time was {:.2f}".format(taxon, min_age, max(times)))
-                    times2 = get_new_branching_times(node, taxon_node, tree, tyoung=min_age, min_ccp=min_ccp, num_new_times=1)
+                    logger.info(
+                        f"    {taxon}: has a minimum age constraint {min_age:.2f} but oldest generated time was {max(times):.2f}"
+                    )
+                    times2 = get_new_branching_times(
+                        node, taxon_node, tree, tyoung=min_age, min_ccp=min_ccp, num_new_times=1
+                    )
                     # Drop the oldest time and add on our new time on the stem lineage
                     times.sort()
                     times.pop()
@@ -519,7 +575,7 @@ For more details, run:
             raise ValueError("Enough species are present but mismatched?")
 
         # Taxon spray
-        logger.info("    {}: adding {} new species".format(taxon, len(species.difference(extant_species))))
+        logger.info(f"    {taxon}: adding {len(species.difference(extant_species))} new species")
         node = fastmrca.get(extant_species)
         times = get_new_branching_times(node, taxon_node, tree, tyoung=get_min_age(node), min_ccp=min_ccp)
         node = fill_new_taxa(tn, node, species.difference(tree_tips), times, ccp < min_ccp)
@@ -532,7 +588,7 @@ For more details, run:
             raise ValueError("Tree is not binary!")
         bar_update()
 
-    assert(is_binary(tree.seed_node))
+    assert is_binary(tree.seed_node)
     # Reset terminal because we aren't using the context manager
     bar.render_finish()
     tree.ladderize()
@@ -541,5 +597,5 @@ For more details, run:
     print()
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     main()
